#!/bin/sh
#########################################################################################
# This is a graphics drivers auto switch script.                                        #
# IGD ----->S3G										#
# PEG------>NVIDIA(free driver or non free driver)					#
# PEG------>ATI (free driver or fglrx driver)						#
#                                                                                       #
# Date:2017-09-14                                                                       #
# Add :fglrx drivers detect.                                                            #
# Add : ZXD detect.                                                                     #
# By: Xunian Tang                                                                       #
#                                                                                       #
# Add ZXE detect,2019-01-18,by txn                                                      #
#########################################################################################

LOG_VIDEO_SWITCH="/var/log/video-switch.log"
#LOG_VIDEO_SWITCH="/tmp/video-switch.log"

ati_fix_xorg_error()
{
ln -sf /usr/lib  /usr/lib64
mkdir -p /usr/X11R6/lib64/modules/
ln -sf /usr/lib/dri  /usr/X11R6/lib64/modules/dri
}

unset_ati_fix_xorg_error()
{
rm -f /usr/lib64 /usr/X11R6/lib64/modules/dri
rm -rf /usr/X11R6/lib64/modules/
}


#2017-07月通过cpu厂家判定ZX平台:vendor id :shanghai 或 CentaurHauls
#2019-01月兆芯E平台同样可通过cpu厂家判定ZX平台:vendor id :Shanghai 或 CentaurHauls
CPU_VENDOR=`cat /proc/cpuinfo | grep -i "vendor_id" | sort | uniq `
echo $CPU_VENDOR >> $LOG_VIDEO_SWITCH  #record info
is_chZX=`echo $CPU_VENDOR | grep -i "CentaurHauls"`
is_shhaiZX=`echo $CPU_VENDOR | grep -i "shanghai"`
if [ ! -n "$is_chZX" ];then
	if [ ! -n "$is_shhaiZX" ];then
		#Is other X86 platform.
		echo " is other x86.platform, exit sh 0" >> $LOG_VIDEO_SWITCH  #record info
		exit 0
	fi
fi
#iso启动时
if [ -d /rofs ];then exit 0; fi

#第一：检测目前显示连接哪张显卡，则默认安装那张显卡驱动
#第二：再根据vendor Id判断显卡类型属于nvidia,ait或via
#第三: ZXD机型则需要判断显卡的设备id，否则无法与ZXA/ZXC等区别
#其它帮助:
#      Vendor id 为0x10de "nVidia Corporation"
#      Vendor id 为0X1002 为[AMD/ATI]
#      Vendor id 为0X1106 为ZXA/ZXC
#                  其中cf03笔记本集显设备id:3a01
#      Vendor id 为0X1d17 为ZXD台式
#                  其中ZXD台式集显设备id:3a03
#2019.1.18兆芯E台式集成显卡:
#     Vendor id为0x1d17
#     device id为0x3a04
#

#hwif="/tmp/hwif.txt"
#hwinfo --gfxcard  > $hwif

################# 根据boot_vga的值来确定哪块显卡是primary #############
hwif="/tmp/hwif.txt"
hwinf1="/tmp/hwinfo1.txt"
hwinf2="/tmp/hwinfo2.txt"

GCARD1_PATH=""
GCARD1_VENDOR=""
GCARD1_DEVICE=""
GCARD2_PATH=""
GCARD2_VENDOR=""
GCARD2_DEVICE=""
card_vendor=""
card_id=""

rm -rf $hwif $hwinf1 $hwinf2
hwinfo --gfxcard > $hwif
CARDNUMBER=`cat $hwif | grep "SysFS ID:" | wc -l`
if [ $CARDNUMBER -eq 1 -o $CARDNUMBER -eq 2 ]; then
	#处理设备信息
	grep -E " SysFS ID: | Vendor: | Device: " $hwif > $hwinf1
	sed -i 's/^[ \t]*//g' $hwinf1 #去掉行首空格

	cat $hwinf1 | awk -F " " '{print $3}' > $hwinf2
#	while read -r line
#	do
#		echo $line | awk -F " " '{print $3}' >> $hwinf2
#	done < $hwinf1
else
	echo "Error"
	exit 0
fi
sed -i 's/0x//g' $hwinf2

FILENUMBER=`cat $hwinf2 | wc -l` #获取文件行数
if [ $CARDNUMBER -eq 1 -a $FILENUMBER -eq 3 ]; then
	GCARD1_PATH=`sed -n '1p' $hwinf2`
	GCARD1_VENDOR=`sed -n '2p' $hwinf2`
	GCARD1_DEVICE=`sed -n '3p' $hwinf2`

	#GCARD1_FILEPATH=`echo "/sys"${GCARD1_PATH}"/boot_vga"`
	#GCARD1_BOOT=`cat $GCARD1_FILEPATH`
	#if [ $GCARD1_BOOT -eq 1 ]; then
		card_vendor=$GCARD1_VENDOR
		card_id=$GCARD1_DEVICE
	#fi
fi
if [ $CARDNUMBER -eq 2 -a $FILENUMBER -eq 6 ]; then
	GCARD1_PATH=`sed -n '1p' $hwinf2`
	GCARD1_VENDOR=`sed -n '2p' $hwinf2`
	GCARD1_DEVICE=`sed -n '3p' $hwinf2`

	GCARD2_PATH=`sed -n '4p' $hwinf2`
	GCARD2_VENDOR=`sed -n '5p' $hwinf2`
	GCARD2_DEVICE=`sed -n '6p' $hwinf2`

	GCARD1_FILEPATH=`echo "/sys"${GCARD1_PATH}"/boot_vga"`
	GCARD1_BOOT=`cat $GCARD1_FILEPATH`
	GCARD2_FILEPATH=`echo "/sys"${GCARD2_PATH}"/boot_vga"`
	GCARD2_BOOT=`cat $GCARD2_FILEPATH`
	if [ $GCARD1_BOOT -eq 1 ]; then
		card_vendor=$GCARD1_VENDOR
		card_id=$GCARD1_DEVICE
	fi
	if [ $GCARD2_BOOT -eq 1 ]; then
		card_vendor=$GCARD2_VENDOR
		card_id=$GCARD2_DEVICE
	fi
fi
############ end 根据boot_vga的值来确定哪块显卡是primary #################

#primary=`cat $hwif | grep -i "Primary display adapter" | awk -F "#" '{print $2}'`
#vgaid=`cat $hwif | grep -i "VGA" | grep $primary`
#card_vendor=`cat $hwif| grep -A20 "$vgaid" | grep -i "Vendor:" | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g' | grep "^Vendor" | awk -F "0x" '{print $2}' | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g'`
#card_id=`cat $hwif| grep -A20 "$vgaid" | grep -i "Device:" | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g' | grep "^Device" | awk -F "0x" '{print $2}' | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g'`
#card_drv=`cat $hwif| grep -A20 "$vgaid" | grep -i "Driver:" `
#card_drv_module=`cat $hwif| grep -A20 "$vgaid" | grep -i "Driver Modules" `

#card_id card_drv card_drv_module 暂时未用
echo " $card_vendor" >> $LOG_VIDEO_SWITCH
echo " $card_id" >> $LOG_VIDEO_SWITCH
#echo $card_drv
#echo $card_drv_module
#内核版本获取
KERNEL_VERSION=`uname -r`

#For 1.5 OS
ZXD_S3G=$(dpkg -l | grep -i "zhaoxin-.*linux-graphics-driver" | awk '{print $2}')
ZXD_S3G_FLAG=$(dpkg -l | grep -i "zhaoxin-.*linux-graphics-driver" | awk '{print $1}')

#For 3.0 OS
ZXD_30_S3G=$(dpkg -l | grep -i "zhaoxin-chx001-linux-graphics-driver" | awk '{print $2}')
ZXD_30_S3G_FLAG=$(dpkg -l | grep -i "zhaoxin-chx001-linux-graphics-driver" | awk '{print $1}')

ZXC_30_S3G=$(dpkg -l | grep -i "xserver-xorg-video-s3g" | awk '{print $2}')
ZXC_30_S3G_FLAG=$(dpkg -l | grep -i "xserver-xorg-video-s3g" | awk '{print $1}')

#for ZXE 3.1 OS status
ZXE_30_S3G=$(dpkg -l | grep -i "zhaoxin-linux-graphics-driver" | awk '{print $2}')
ZXE_30_S3G_FLAG=$(dpkg -l | grep -i "zhaoxin-linux-graphics-driver" | awk '{print $1}')
#end ZXE 3.1 OS status

#由于时间紧迫，先复用早期代码，毕竟早期代码经历2～3年的验证，后续可逐步完善
#
NVIDIA_NAME=$(dpkg -l | grep nvidia  | grep "binary" | awk '{print $2}')
NVIDIA_FLAG=$(dpkg -l | grep nvidia  | grep "binary" | awk '{print $1}')
S3G_NAME=$(dpkg -l | grep s3-linux-graphics-driver | awk '{print $2}')
S3G_FLAG=$(dpkg -l | grep s3-linux-graphics-driver | awk '{print $1}')
ATI_NAME=$(dpkg -l | grep xserver-xorg-video-ati | awk '{print $2}')

# fglrx drivers status
FGLRX_NAME=$(dpkg -l | grep "fglrx" | awk '{print $2}' | grep -x fglrx)
FGLRX_FLAG=$(dpkg -l | grep "fglrx" | awk '{print $1,$2}' | grep -x "ii fglrx" | awk '{print $1}')
FGLRX_AMDCCCLE_NAME=$(dpkg -l | grep "fglrx-amdcccle" | awk '{print $2}')
FGLRX_AMDCCCLE_FLAG=$(dpkg -l | grep "fglrx-amdcccle" | awk '{print $1}')
FGLRX_CORE_NAME=$(dpkg -l | grep "fglrx-core" | awk '{print $2}')
FGLRX_CORE_FLAG=$(dpkg -l | grep "fglrx-core" | awk '{print $1}')
FGLRX_DEV_NAME=$(dpkg -l | grep "fglrx-dev" | awk '{print $2}')
FGLRX_DEV_FLAG=$(dpkg -l | grep "fglrx-dev" | awk '{print $1}')

LOG_VIDEO_S3G="/var/log/video-s3g.log"
LOG_VIDEO_NVIDIA="/var/log/video-nvidia.log"
LOG_VIDEO_ATI="/var/log/video-ati.log"
LOG_VIDEO_OTHER="/var/log/video-other.log"
LOG_VIDEO_WARN="/var/log/video-warn.log"

#purge zx s3g drivers.
purge_zx_s3g_drivers()
{
	if [ -n "${S3G_NAME}" ]; then
		echo " remove s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${S3G_NAME} >> $LOG_VIDEO_S3G    2>&1
	fi
	if [ -n "${ZXD_S3G}" ]; then
		echo " remove zxd s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxd s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXD_S3G} >> $LOG_VIDEO_S3G    2>&1
	fi
	#For 3.0 OS
	if [ -n "${ZXD_30_S3G}" ]; then
		echo " remove zxd s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxd s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXD_30_S3G} >> $LOG_VIDEO_S3G    2>&1
	fi
	if [ -n "${ZXC_30_S3G}" ]; then
		echo " remove zxc+3.0 s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxc+3.0 s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXC_30_S3G} >> $LOG_VIDEO_S3G    2>&1
	fi
	#For zhaoxin E
	if [ -n "${ZXE_30_S3G}" ]; then
		echo " remove zxe s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxe s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXE_30_S3G} >> $LOG_VIDEO_S3G    2>&1
	fi

	rm -f /etc/X11/xorg.conf*
	rm -f /etc/X11/via_kernel.conf*
	rm -f /etc/X11/viaXconfig.conf*
	
}

#purge all nvidia-346
purge_nvidia_drivers()
{
	if [ -n "${NVIDIA_NAME}" ]; then
		echo " remove nvidia driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall nvidia driver-----------------------: `date` " >>    $LOG_VIDEO_NVIDIA
		dpkg -P nvidia-346  >> $LOG_VIDEO_NVIDIA    2>&1
	fi
}

#purge all fglrx drivers.
purge_fglrx_drivers()
{
	if [ -n "${FGLRX_NAME}" ] || [ -n "${FGLRX_AMDCCCLE_NAME}" ] || [ -n "${FGLRX_CORE_NAME}" ] || [ -n "${FGLRX_DEV_NAME}" ]; then
		echo " start to purge fglrx drivers:`date` "  >> $LOG_VIDEO_SWITCH
		echo "Start to purge fglrx drivers:`date` "  >> $1
		dpkg -P ${FGLRX_NAME} ${FGLRX_AMDCCCLE_NAME} ${FGLRX_CORE_NAME} ${FGLRX_DEV_NAME} >> $1   2>&1
		echo " purge fglrx drivers done:`date` "  >> $LOG_VIDEO_SWITCH
		unset_ati_fix_xorg_error
	fi
}

#reboot system at the first time, after switch graphics driver.
reboot_system()
{
	echo " reboot system now!" >> $LOG_VIDEO_SWITCH
	reboot
}

#For 1.5 & 2.0 OS
check_install_s3g_drv()
{
	#s3g card
	echo " Use s3g card =`date` "  >> $LOG_VIDEO_SWITCH
	#check s3g drivers files type
	s3g_files=$(find /usr/local/nfsdrivers/ -name "s3-linux-graphics-driver_*.deb"  -type f )
	#check s3g files status
	s3g_file_status=$(echo $s3g_files | grep "s3-linux-graphics-driver_*" )
	if [ -n "${s3g_file_status}" ];then
		#now /usr/local/nfs.../ has s3-xxx.deb
		#checkout system s3g dirvers status.
		if [ ! -n "${S3G_NAME}" ] || [ -n "${S3G_NAME}" -a "${S3G_FLAG}" != "ii" ]; then
			service mdm stop
			purge_nvidia_drivers
			#purge ati drivers. dpkg -P fglrx.xxx (3-5)
			purge_fglrx_drivers $LOG_VIDEO_ATI
			purge_zx_s3g_drivers

			echo " install s3g driver " >> $LOG_VIDEO_SWITCH
			echo "-------------------install s3g driver-----------------------: `date` " >>            $LOG_VIDEO_S3G
			dpkg -i /usr/local/nfsdrivers/s3-linux-graphics-driver_*.deb >> $LOG_VIDEO_S3G   2>&1
			#repeat install s3g until success, at most 3 times
			S3G_FLAG=$(dpkg -l | grep s3-linux-graphics-driver | awk '{print$1}')
			while [ "${S3G_FLAG}" != "ii" ]; do
				purge_zx_s3g_drivers
				echo "-------------------install s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
				dpkg -i /usr/local/nfsdrivers/s3-linux-graphics-driver_*.deb >> $LOG_VIDEO_S3G   2>&1
				S3G_FLAG=$(dpkg -l | grep s3-linux-graphics-driver | awk '{print$1}')
				k=`expr $k + 1`
				echo "Repeat install s3g driver $k times already: `date` " >> $LOG_VIDEO_SWITCH
				if [ $k -gt 2 ]; then
					echo "Repeat s3g 3 times and enter the system. Now the s3g status is ${S3G_FLAG}: `date` " >> $LOG_VIDEO_SWITCH
					break
				fi
			done
			update-alternatives --set x86_64-linux-gnu_gl_conf /usr/lib/s3g/ld.so.conf
			ldconfig

			if [ ! -e /etc/X11/xorg.conf ]; then
				echo " File xorg.conf had change, rm xorg.conf.* : `date` " >> $LOG_VIDEO_SWITCH
				rm -f /etc/X11/xorg.conf.*
				rm -f /etc/X11/via_kernel.conf.*
				rm -f /etc/X11/viaXconfig.conf.*
				cp -f /usr/share/s3g/* /etc/X11/
				echo " cp all s3g conf backto 'etc/X11': `date` " >> $LOG_VIDEO_SWITCH
			fi
			echo " install s3g driver done: `date` " >> $LOG_VIDEO_SWITCH
			#reboot system to detect video graph drivers.
			#otherwise 'service mdm restart', kernel detect the other video drivers at this time.
			reboot_system
		else
			echo "---> still s3g <----: `date` "  >> $LOG_VIDEO_SWITCH
			if [ ! -e /etc/X11/xorg.conf ]; then
				echo " File xorg.conf had change, rm xorg.conf.* : `date` " >> $LOG_VIDEO_SWITCH
				rm -f /etc/X11/xorg.conf.*
				rm -f /etc/X11/via_kernel.conf.*
				rm -f /etc/X11/viaXconfig.conf.*
				cp -f /usr/share/s3g/* /etc/X11/
				echo " cp all s3g conf backto 'etc/X11': `date` " >> $LOG_VIDEO_SWITCH
				service mdm restart
			fi
		fi #end system s3g drivers status checkout.
	else
		#if have fglrx or nvidia ,then purge it
		purge_fglrx_drivers $LOG_VIDEO_OTHER
		purge_nvidia_drivers
	fi #end checkout s3g files
}

#For 1.5 OS
check_install_nvidia_drv()
{
	#NVIDIA card
	echo " Use nvidia card =`date` "  >> $LOG_VIDEO_SWITCH
	#3.xx内核支持,4.3.xx不支持
	#check nvidia drivers files type
	nvidia_files=$(find /usr/local/nfsdrivers/ -name "nvidia*.deb"  -type f )
	#check nvidia files status
	nvidia_file_status=$(echo $nvidia_files | grep "nvidia-[0-9][0-9]*" )
	if [ -n "${nvidia_file_status}" ];then
		#now /usr/local/nfs.../ has nvidia-xxx.deb
		#checkout system nvidia dirvers status.
		if [ ! -n "${NVIDIA_NAME}" ] || [ -n "${NVIDIA_NAME}" -a "${NVIDIA_FLAG}" != "ii" ]; then #bug
			service mdm stop
			purge_zx_s3g_drivers
			#purge ati drivers. dpkg -P fglrx.xxx (3-5)
			purge_fglrx_drivers $LOG_VIDEO_NVIDIA

			echo " install nvidia driver: `date` " >> $LOG_VIDEO_SWITCH
			echo "-------------------install nvidia driver-----------------------: `date` " >>           $LOG_VIDEO_NVIDIA
			dpkg -i /usr/local/nfsdrivers/nvidia-346_*.deb >>      $LOG_VIDEO_NVIDIA   2>&1
			#repeat install nvidia until success, at most 3 times
			NVIDIA_FLAG=$(dpkg -l | grep nvidia  | grep "binary" | awk '{print$1}')
			while [ "${NVIDIA_FLAG}" != "ii" ]; do
				echo "-------------------install nvidia driver-----------------------: `date` " >>   $LOG_VIDEO_NVIDIA
				dpkg -i /usr/local/nfsdrivers/nvidia-346_*.deb >> $LOG_VIDEO_NVIDIA    2>&1
				NVIDIA_FLAG=$(dpkg -l | grep nvidia  | grep "binary" | awk '{print$1}')
				k=`expr $k + 1`
				echo "Repeat install nvidia driver $k times already: `date` " >> $LOG_VIDEO_SWITCH
				if [ $k -gt 2 ]; then
					echo "Repeat nvidia 3 times and enter the system. Now the nvidia status is ${NVIDIA_FLAG}.: `date` " >> $LOG_VIDEO_SWITCH
					break
				fi
			done
			echo " install nvidia driver done: `date` " >> $LOG_VIDEO_SWITCH
			#reboot system to detect graphics drivers.
			#otherwise 'service mdm restart', kernel detect the other video drivers at this time.
			reboot_system
		fi #驱动安装判断
	else
		#if have fglrx or s3g ,purge it
		purge_fglrx_drivers  $LOG_VIDEO_OTHER
		purge_zx_s3g_drivers
	fi #nvidia dirvers deb check
}

#For 1.5 & 2.0 OS
check_install_ati_drv()
{
	#Start ati card detect. if have fglrx.xxx drives then use it, else detect the open drivers(radoen.xxx)
	echo " Use ati card :`date` "  >> $LOG_VIDEO_SWITCH
	#check fglrx files type
	fglrx_files=$(find /usr/local/nfsdrivers/ -name "fglrx*.deb"  -type f )
	#check fglrx_15.xxx file status(just one file status:fglrx_xxx.deb)
	fglrx_file_status=$(echo $fglrx_files | grep "fglrx_[0-9][0-9]*" )
	if [ -n "${fglrx_file_status}" ];then
		#now /usr/local/xxxdrivers/ dir have fglrx.xxx.deb
		#check all fglrx drivers status in the system,by dpkg command
		if [ ! -n "${FGLRX_NAME}" ] || [ -n "${FGLRX_NAME}" -a "${FGLRX_FLAG}" != "ii" ] || [ ! -n "${FGLRX_AMDCCCLE_NAME}" ] || [ -n "${FGLRX_AMDCCCLE_NAME}" -a "${FGLRX_AMDCCCLE_FLAG}" != "ii" ] || [ ! -n "${FGLRX_CORE_NAME}" ] || [ -n "${FGLRX_CORE_NAME}" -a "${FGLRX_CORE_FLAG}" != "ii" ]; then
			echo " fglrx.xxx has not install or install error at last time. " >> $LOG_VIDEO_SWITCH
			service mdm stop
			#If have nvidia drivers,then purge it
			purge_nvidia_drivers

			#If have s3g drivers,then purge it
			purge_zx_s3g_drivers

			#Install fglrx.xxx drivers
			echo " install fglrx drivers: `date` " >> $LOG_VIDEO_SWITCH
			echo "-------------------install fglrx drivers-----------------------: `date` " >>           $LOG_VIDEO_ATI
			dpkg -i /usr/local/nfsdrivers/fglrx*.deb >>      $LOG_VIDEO_ATI   2>&1

			ati_fix_xorg_error

			#checkout fglrx.xxx drivers status, at most 3 times.
			FGLRX_DEBS_STA=$(dpkg -l | grep fglrx |awk '{if($1 != "ii") print $1, $2}')
			while [ -n "${FGLRX_DEBS_STA}" ]; do
				echo " repeat install fglrx.xxx!" >> $LOG_VIDEO_SWITCH
				dpkg -i /usr/local/nfsdrivers/fglrx*.deb >>      $LOG_VIDEO_ATI  2>&1
				FGLRX_DEBS_STA=$(dpkg -l | grep fglrx |awk '{if($1 != "ii") print $1, $2}')

				k=`expr $k + 1`
				if [ $k -gt 2 ]; then
					break
				fi
			done

			echo " install fglrx drivers done: `date` " >> $LOG_VIDEO_SWITCH
			#reboot system to detect graphics drivers.
			#otherwise 'service mdm restart', kernel detect the other video drivers at this time.
			reboot_system
		else
			echo " fglrx had install" >> $LOG_VIDEO_SWITCH
		fi
	else
		echo " no fglrx debs found" >> $LOG_VIDEO_SWITCH
		purge_zx_s3g_drivers
		purge_nvidia_drivers
	fi
}

other_card_purge_drv()
{
	#other
	echo "Use the other card,(no ati,nv,s3g found): `date` " >> $LOG_VIDEO_SWITCH
	purge_zx_s3g_drivers
	purge_nvidia_drivers
	#purge ati drivers. dpkg -P fglrx.xxx (3-5)
	purge_fglrx_drivers  $LOG_VIDEO_OTHER
}

#3.0 OS 桌面是ligtdm, 1.5&2.0是mdm
stop_mdm_lightdm()
{
	case $KERNEL_VERSION in
		3.19* | 4.3*)
		#OS 1.5 & 2.0
		echo "$KERNEL_VERSION, service mdm stop ..." >> $LOG_VIDEO_SWITCH
		service mdm stop
		;;

		4.9*)
		#OS 3.0
		echo "$KERNEL_VERSION, service lightdm stop ..." >> $LOG_VIDEO_SWITCH
		service lightdm stop
		;;

		*)
		echo "Other kernel version" >> $LOG_VIDEO_OTHER
		;;
	esac
}

#该函数用于在当前没有对应的显卡驱动情况下，处理之前安装过的显卡驱动，并重启。否则有可能出现桌面崩溃情况
#而且reboot写在条件内，否则死循环.
proprietary_drivers_purge()
{
	#purge s3g driver
	if [ -n "${S3G_NAME}" ]; then
		stop_mdm_lightdm
		echo " remove s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${S3G_NAME} >> $LOG_VIDEO_S3G    2>&1
		rm -f /etc/X11/xorg.conf*
		rm -f /etc/X11/via_kernel.conf*
		rm -f /etc/X11/viaXconfig.conf*
		reboot_system
	fi
	#purge zhaoxin driver , For 1.5 OS
	if [ -n "${ZXD_S3G}" ]; then
		stop_mdm_lightdm
		echo " remove zxd s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxd s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXD_S3G} >> $LOG_VIDEO_S3G    2>&1
		rm -f /etc/X11/xorg.conf*
		rm -f /etc/X11/via_kernel.conf*
		rm -f /etc/X11/viaXconfig.conf*
		reboot_system
	fi
	#purge zhaoxin driver , For 3.0 OS
	if [ -n "${ZXD_30_S3G}" ]; then
		stop_mdm_lightdm
		echo " remove zxd s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxd s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXD_30_S3G} >> $LOG_VIDEO_S3G    2>&1
		rm -f /etc/X11/xorg.conf*
		rm -f /etc/X11/via_kernel.conf*
		rm -f /etc/X11/viaXconfig.conf*
		reboot_system
	fi
	#purge zxc+3.0 xserver-video-s3g driver
	if [ -n "${ZXC_30_S3G}" ]; then
		stop_mdm_lightdm
		echo " remove zxc+3.0 s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxc+3.0 s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXC_30_S3G} >> $LOG_VIDEO_S3G    2>&1
		rm -f /etc/X11/xorg.conf*
		rm -f /etc/X11/via_kernel.conf*
		rm -f /etc/X11/viaXconfig.conf*
		reboot_system
	fi
	#purge nvidia driver
	if [ -n "${NVIDIA_NAME}" ]; then
		stop_mdm_lightdm
		echo " remove nvidia driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall nvidia driver-----------------------: `date` " >>    $LOG_VIDEO_NVIDIA
		dpkg -P nvidia-346  >> $LOG_VIDEO_NVIDIA    2>&1
		reboot_system
	fi
	#purge fglrx driver
	if [ -n "${FGLRX_NAME}" ] || [ -n "${FGLRX_AMDCCCLE_NAME}" ] || [ -n "${FGLRX_CORE_NAME}" ] || [ -n "${FGLRX_DEV_NAME}" ]; then
		stop_mdm_lightdm
		echo " start to purge fglrx drivers:`date` "  >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall fglrx driver-----------------------: `date` " >>    $LOG_VIDEO_ATI
		dpkg -P ${FGLRX_NAME} ${FGLRX_AMDCCCLE_NAME} ${FGLRX_CORE_NAME} ${FGLRX_DEV_NAME} >> $1   2>&1
		echo " purge fglrx drivers done:`date` "  >> $LOG_VIDEO_SWITCH
		reboot_system
	fi
	#purge ZXE drivers
	if [ -n "${ZXE_30_S3G}" ]; then
		stop_mdm_lightdm
		echo " remove zxe s3g driver: `date`  " >> $LOG_VIDEO_SWITCH
		echo "-------------------Uninstall zxe s3g driver-----------------------: `date` " >>    $LOG_VIDEO_S3G
		dpkg -P ${ZXE_30_S3G} >> $LOG_VIDEO_S3G    2>&1
		rm -f /etc/X11/xorg.conf*
		rm -f /etc/X11/via_kernel.conf*
		rm -f /etc/X11/viaXconfig.conf*
		reboot_system
	fi
}
#end 早期编写代码

#ZXD机型集成显卡驱动安装和卸载代码 (For 1.5 OS)
install_ZXD_s3g_drv()
{
	echo " Use ZXD s3g card =`date` "  >> $LOG_VIDEO_SWITCH
	#check s3g drivers files type
	s3g_files=$(find /usr/local/nfsdrivers/ -name "zhaoxin-*linux-graphics-driver_*.deb"  -type f )
	#check s3g files status
	s3g_file_status=$(echo $s3g_files | grep "zhaoxin-.*linux-graphics-driver_.*" )
	if [ -n "${s3g_file_status}" ];then
		#now /usr/local/nfs.../ has s3-xxx.deb
		#checkout system s3g dirvers status.
		if [ ! -n "${ZXD_S3G}" ] || [ -n "${ZXD_S3G}" -a "${ZXD_S3G_FLAG}" != "ii" ]; then
			#service mdm stop
			stop_mdm_lightdm

			purge_nvidia_drivers
			#purge ati drivers. dpkg -P fglrx.xxx (3-5)
			purge_fglrx_drivers $LOG_VIDEO_S3G
			purge_zx_s3g_drivers

			echo " install zxd s3g driver " >> $LOG_VIDEO_SWITCH
			echo "-------------------install zxd s3g driver-----------------------: `date` " >>            $LOG_VIDEO_S3G
			dpkg -i /usr/local/nfsdrivers/zhaoxin-*linux-graphics-driver_*.deb >> $LOG_VIDEO_S3G   2>&1
			#repeat install s3g until success, at most 3 times
			ZXD_S3G_FLAG=$(dpkg -l | grep "zhaoxin-.*linux-graphics-driver" | awk '{print$1}')
			while [ "${ZXD_S3G_FLAG}" != "ii" ]; do
				purge_zx_s3g_drivers
				echo "-------------------install zxd s3g driver --------------------: `date` " >>    $LOG_VIDEO_S3G
				dpkg -i /usr/local/nfsdrivers/zhaoxin-*linux-graphics-driver_*.deb >> $LOG_VIDEO_S3G   2>&1
				ZXD_S3G_FLAG=$(dpkg -l | grep "zhaoxin-.*linux-graphics-driver" | awk '{print$1}')
				k=`expr $k + 1`
				echo "Repeat install zxd s3g driver $k times already: `date` " >> $LOG_VIDEO_SWITCH
				if [ $k -gt 2 ]; then
					echo "Repeat s3g 3 times and enter the system. Now the zxd s3g status is ${ZXD_S3G_FLAG}: `date` " >> $LOG_VIDEO_SWITCH
					break
				fi
			done
			#update-alternatives --set x86_64-linux-gnu_gl_conf /usr/lib/s3g/ld.so.conf
			#ldconfig

			#if [ ! -e /etc/X11/xorg.conf ]; then
			#	echo " File xorg.conf had change, rm xorg.conf.* : `date` " >> $LOG_VIDEO_SWITCH
			#	rm -f /etc/X11/xorg.conf.*
			#	rm -f /etc/X11/via_kernel.conf.*
			#	rm -f /etc/X11/viaXconfig.conf.*
			#	cp -f /usr/share/s3g/* /etc/X11/
			#	echo " cp all s3g conf backto 'etc/X11': `date` " >> $LOG_VIDEO_SWITCH
			#fi
			echo " install zxd s3g driver done: `date` " >> $LOG_VIDEO_SWITCH
			#reboot system to detect video graph drivers.
			#otherwise 'service mdm restart', kernel detect the other video drivers at this time.
			reboot_system
		else
			echo "---> still zxd s3g <----: `date` "  >> $LOG_VIDEO_SWITCH
			#if [ ! -e /etc/X11/xorg.conf ]; then
			#	echo " File xorg.conf had change, rm xorg.conf.* : `date` " >> $LOG_VIDEO_SWITCH
			#	rm -f /etc/X11/xorg.conf.*
			#	rm -f /etc/X11/via_kernel.conf.*
			#	rm -f /etc/X11/viaXconfig.conf.*
			#	cp -f /usr/share/s3g/* /etc/X11/
			#	echo " cp all s3g conf backto 'etc/X11': `date` " >> $LOG_VIDEO_SWITCH
			#	service mdm restart
			#fi
		fi #end system s3g drivers status checkout.
	else
		#if have fglrx or nvidia ,then purge it
		purge_fglrx_drivers  $LOG_VIDEO_OTHER
		purge_nvidia_drivers
	fi #end checkout s3g files
}

#For 3.0 OS
install_ZXD_30_s3g_drv()
{
	echo " Use ZXD s3g card =`date` "  >> $LOG_VIDEO_SWITCH
	#check s3g drivers files type
	###2018-0409 去除'-30'，为兼容"-30"而修改
	s3g_files=$(find /usr/local/nfsdrivers/ -name "zhaoxin-chx001-linux-graphics-driver*.deb"  -type f )
	#check s3g files status
	s3g_file_status=$(echo $s3g_files | grep "zhaoxin-chx001-linux-graphics-driver.*.deb" )
	if [ -n "${s3g_file_status}" ];then
		#now /usr/local/nfs.../ has s3-xxx.deb
		#checkout system s3g dirvers status.
		if [ ! -n "${ZXD_30_S3G}" ] || [ -n "${ZXD_30_S3G}" -a "${ZXD_30_S3G_FLAG}" != "ii" ]; then
			#service mdm stop
			stop_mdm_lightdm

			purge_nvidia_drivers
			#purge ati drivers. dpkg -P fglrx.xxx (3-5)
			purge_fglrx_drivers $LOG_VIDEO_S3G
			purge_zx_s3g_drivers

			echo " install zxd s3g 3.0 driver " >> $LOG_VIDEO_SWITCH
			echo "-------------------install zxd s3g 3.0 driver-----------------------: `date` " >>            $LOG_VIDEO_S3G
			dpkg -i /usr/local/nfsdrivers/zhaoxin-chx001-linux-graphics-driver*.deb >> $LOG_VIDEO_S3G   2>&1
			#repeat install s3g until success, at most 3 times
			ZXD_S3G_FLAG=$(dpkg -l | grep "zhaoxin-chx001-linux-graphics-driver" | awk '{print$1}')
			while [ "${ZXD_S3G_FLAG}" != "ii" ]; do
				purge_zx_s3g_drivers
				echo "-------------------install zxd s3g 3.0 driver --------------------: `date` " >>    $LOG_VIDEO_S3G
				dpkg -i /usr/local/nfsdrivers/zhaoxin-chx001-linux-graphics-driver*.deb >> $LOG_VIDEO_S3G   2>&1
				ZXD_S3G_FLAG=$(dpkg -l | grep "zhaoxin-chx001-linux-graphics-driver" | awk '{print$1}')
				k=`expr $k + 1`
				echo "Repeat install zxd s3g 3.0 driver $k times already: `date` " >> $LOG_VIDEO_SWITCH
				if [ $k -gt 2 ]; then
					echo "Repeat s3g 3 times and enter the system. Now the zxd s3g status is ${ZXD_S3G_FLAG}: `date` " >> $LOG_VIDEO_SWITCH
					break
				fi
			done
			#update-alternatives --set x86_64-linux-gnu_gl_conf /usr/lib/s3g/ld.so.conf
			#ldconfig

			#if [ ! -e /etc/X11/xorg.conf ]; then
			#	echo " File xorg.conf had change, rm xorg.conf.* : `date` " >> $LOG_VIDEO_SWITCH
			#	rm -f /etc/X11/xorg.conf.*
			#	rm -f /etc/X11/via_kernel.conf.*
			#	rm -f /etc/X11/viaXconfig.conf.*
			#	cp -f /usr/share/s3g/* /etc/X11/
			#	echo " cp all s3g conf backto 'etc/X11': `date` " >> $LOG_VIDEO_SWITCH
			#fi
			echo " install zxd s3g 3.0 driver done: `date` " >> $LOG_VIDEO_SWITCH
			#reboot system to detect video graph drivers.
			#otherwise 'service mdm restart', kernel detect the other video drivers at this time.
			reboot_system
		else
			echo "---> still zxd s3g 3.0 <----: `date` "  >> $LOG_VIDEO_SWITCH
			#if [ ! -e /etc/X11/xorg.conf ]; then
			#	echo " File xorg.conf had change, rm xorg.conf.* : `date` " >> $LOG_VIDEO_SWITCH
			#	rm -f /etc/X11/xorg.conf.*
			#	rm -f /etc/X11/via_kernel.conf.*
			#	rm -f /etc/X11/viaXconfig.conf.*
			#	cp -f /usr/share/s3g/* /etc/X11/
			#	echo " cp all s3g conf backto 'etc/X11': `date` " >> $LOG_VIDEO_SWITCH
			#	service mdm restart
			#fi
		fi #end system s3g drivers status checkout.
	else
		#if have fglrx or nvidia ,then purge it
		purge_fglrx_drivers  $LOG_VIDEO_OTHER
		purge_nvidia_drivers
	fi #end checkout s3g files
}
#end ZXD机型集成显卡驱动安装和卸载

##################################################
#For ZXE + 3.1 OS
#根据与兆芯厂家约定：ZX-E后续发布版本会修改为:zhaoxin-linux-graphics-driver的deb包名
install_ZXE_30_s3g_drv()
{
	echo " Use ZXE s3g card =`date` "  >> $LOG_VIDEO_SWITCH
	#check s3g drivers files type.
	s3g_files=$(find /usr/local/nfsdrivers/ -name "zhaoxin-linux-graphics-driver*.deb"  -type f )
	#check s3g files status
	s3g_file_status=$(echo $s3g_files | grep "zhaoxin-linux-graphics-driver.*.deb" )
	if [ -n "${s3g_file_status}" ];then
		#now /usr/local/nfs.../ has xxx.deb
		#checkout system s3g dirvers status.
		if [ ! -n "${ZXE_30_S3G}" ] || [ -n "${ZXE_30_S3G}" -a "${ZXE_30_S3G_FLAG}" != "ii" ]; then
			#service mdm stop
			stop_mdm_lightdm

			purge_nvidia_drivers
			#purge ati drivers. dpkg -P fglrx.xxx (3-5)
			purge_fglrx_drivers $LOG_VIDEO_S3G
			purge_zx_s3g_drivers

			echo " install zxe s3g 3.0 driver " >> $LOG_VIDEO_SWITCH
			echo "-------------------install zxe s3g 3.0 driver-----------------------: `date` " >>            $LOG_VIDEO_S3G
			dpkg -i /usr/local/nfsdrivers/zhaoxin-linux-graphics-driver*.deb >> $LOG_VIDEO_S3G   2>&1
			#repeat install s3g until success, at most 3 times
			ZXE_S3G_FLAG=$(dpkg -l | grep "zhaoxin-linux-graphics-driver" | awk '{print$1}')
			while [ "${ZXE_S3G_FLAG}" != "ii" ]; do
				purge_zx_s3g_drivers
				echo "-------------------install zxe s3g 3.0 driver --------------------: `date` " >>    $LOG_VIDEO_S3G
				dpkg -i /usr/local/nfsdrivers/zhaoxin-linux-graphics-driver*.deb >> $LOG_VIDEO_S3G   2>&1
				ZXE_S3G_FLAG=$(dpkg -l | grep "zhaoxin-linux-graphics-driver" | awk '{print$1}')
				k=`expr $k + 1`
				echo "Repeat install zxe s3g 3.0 driver $k times already: `date` " >> $LOG_VIDEO_SWITCH
				if [ $k -gt 2 ]; then
					echo "Repeat s3g 3 times and enter the system. Now the zxe s3g status is ${ZXE_S3G_FLAG}: `date` " >> $LOG_VIDEO_SWITCH
					break
				fi
			done
			#update-alternatives --set x86_64-linux-gnu_gl_conf /usr/lib/s3g/ld.so.conf
			#ldconfig

			echo " install zxe s3g 3.0 driver done: `date` " >> $LOG_VIDEO_SWITCH
			#reboot system to detect video graph drivers.
			#otherwise 'service mdm restart', kernel detect the other video drivers at this time.
			reboot_system
		else
			echo "---> still zxe s3g 3.0 <----: `date` "  >> $LOG_VIDEO_SWITCH
		fi #end system s3g drivers status checkout.
	else
		#if have fglrx or nvidia ,then purge it
		purge_fglrx_drivers  $LOG_VIDEO_OTHER
		purge_nvidia_drivers
	fi #end checkout s3g files
}
#############End For ZXE + 3.1 OS 集成显卡驱动安装

install_ZXC_30_s3g_drv()
{
	echo " ZXC+3.0 s3g card =`date` "  >> $LOG_VIDEO_SWITCH
	#check s3g drivers files type
	s3g_files=$(find /usr/local/nfsdrivers/ -name "xserver-xorg-video-s3g_*.deb"  -type f )
	#check s3g files status
	s3g_file_status=$(echo $s3g_files | grep "xserver-xorg-video-s3g_.*" )
	if [ -n "${s3g_file_status}" ];then
		#now /usr/local/nfs.../ has xserver-xorg-video-s3g_*.deb
		#checkout system s3g dirvers status.
		if [ ! -n "${ZXC_30_S3G}" ] || [ -n "${ZXC_30_S3G}" -a "${ZXC_30_S3G_FLAG}" != "ii" ]; then
			echo "service lightdm stop ... " >> $LOG_VIDEO_S3G
			service lightdm stop
			purge_nvidia_drivers
			purge_fglrx_drivers $LOG_VIDEO_S3G
			purge_zx_s3g_drivers

			echo " install zxc+3.0 s3g driver " >> $LOG_VIDEO_SWITCH
			echo "-------------------install zxc+3.0 s3g driver-----------------------: `date` " >>            $LOG_VIDEO_S3G
			dpkg -i /usr/local/nfsdrivers/xserver-xorg-video-s3g_*.deb >> $LOG_VIDEO_S3G   2>&1
			#repeat install s3g until success, at most 3 times
			ZXC_30_S3G_FLAG=$(dpkg -l | grep "xserver-xorg-video-s3g" | awk '{print$1}')
			while [ "${ZXC_30_S3G_FLAG}" != "ii" ]; do
				purge_zx_s3g_drivers
				echo "-------------------install zxc+3.0 s3g driver --------------------: `date` " >>    $LOG_VIDEO_S3G
				dpkg -i /usr/local/nfsdrivers/xserver-xorg-video-s3g_*.deb >> $LOG_VIDEO_S3G   2>&1
				ZXC_30_S3G_FLAG=$(dpkg -l | grep "xserver-xorg-video-s3g" | awk '{print$1}')
				k=`expr $k + 1`
				echo "Repeat install zxc+3.0 s3g driver $k times already: `date` " >> $LOG_VIDEO_SWITCH
				if [ $k -gt 2 ]; then
					echo "Repeat s3g 3 times and enter the system. Now the zxc+3.0 s3g status is ${ZXC_30_S3G_FLAG}: `date` " >> $LOG_VIDEO_SWITCH
					break
				fi
			done

			echo " install zxc+3.0 s3g driver done: `date` " >> $LOG_VIDEO_SWITCH
			reboot_system
		else
			echo "---> still zxc+3.0 s3g <----: `date` "  >> $LOG_VIDEO_SWITCH
		fi #end system s3g drivers status checkout.
	else
		#if have fglrx or nvidia ,then purge it
		purge_fglrx_drivers  $LOG_VIDEO_OTHER
		purge_nvidia_drivers
	fi #end checkout s3g files
}

#zxa 1.5系统
ZXA_15_install_drv()
{
	echo "ZXA OS VERSION 1.5"  >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		check_install_nvidia_drv
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxa/zxc/zxc+
		1106*)
		check_install_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxa 2.0 系统
ZXA_20_install_drv()
{
	echo "ZXA OS VERSION 2.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia,not support..
		10de*)
		#check_install_nvidia_drv
		proprietary_drivers_purge
		echo "No nvidia driver for ZXA+2.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxa/zxc/zxc+
		1106*)
		check_install_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxa 3.0 系统
ZXA_30_install_drv()
{
	echo "ZXA OS VERSION 3.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXA+3.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		proprietary_drivers_purge
		echo "No amd/ati driver for ZXA+3.0, use Radeon driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is zxa/zxc/zxc+
		1106*)
		install_ZXC_30_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}

#zxc 1.5系统
ZXC_15_install_drv()
{
	echo "ZXC OS VERSION 1.5" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		check_install_nvidia_drv
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxa/zxc/zxc+
		1106*)
		check_install_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxc 2.0系统
ZXC_20_install_drv()
{
	echo "ZXC OS VERSION 2.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia,not support..
		10de*)
		#check_install_nvidia_drv
		proprietary_drivers_purge
		echo "No nvidia driver for ZXC+2.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxa/zxc/zxc+
		1106*)
		check_install_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxc 3.0系统
ZXC_30_install_drv()
{
	echo "ZXC OS VERSION 3.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXC+3.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		proprietary_drivers_purge
		echo "No amd/ati driver for ZXC+3.0, use Radeon driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is zxa/zxc/zxc+
		1106*)
		install_ZXC_30_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}

#zxc+ 1.5系统
ZXCplus_15_install_drv()
{
	echo "ZXC+ OS VERSION 1.5" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		check_install_nvidia_drv
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxa/zxc/zxc+
		1106*)
		check_install_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxc+ 2.0系统
ZXCplus_20_install_drv()
{
	echo "ZXC+ OS VERSION 2.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia,not support..
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXCplus+2.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxa/zxc/zxc+
		1106*)
		check_install_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxc+ 3.0系统
ZXCplus_30_install_drv()
{
	echo "ZXC+ OS VERSION 3.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXCplus+3.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		proprietary_drivers_purge
		echo "No amd/ati driver for ZXCplus+3.0, use Radeon driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is zxa/zxc/zxc+
		1106*)
		install_ZXC_30_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}

#zxd 1.5系统
ZXD_15_install_drv()
{
	echo "ZXD OS VERSION 1.5" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		check_install_nvidia_drv
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxd 集成显卡
		1d17*)
		install_ZXD_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxd 2.0系统
ZXD_20_install_drv()
{
	echo "ZXD OS VERSION 2.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia,not support..
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXD+2.0, use Nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		check_install_ati_drv
		;;

		#is zxd 集成显卡,内核4.3.x暂不支持
		1d17*)
		#install_ZXD_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#zxd 3.0 系统
ZXD_30_install_drv()
{
	echo "ZXD OS VERSION 3.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXD+3.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		proprietary_drivers_purge
		echo "No amd/ati driver for ZXD+3.0, use Radeon driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is zxa/zxc/zxc+
		1d17*)
		install_ZXD_30_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}

#For 3.1 OS ZXE
ZXE_30_install_drv()
{
	echo "ZXE OS VERSION 3.0" >> $LOG_VIDEO_SWITCH
	case $card_vendor in
		#is nvidia
		10de*)
		proprietary_drivers_purge
		echo "No nvidia driver for ZXE+3.0, use nouveau driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is amd/ati
		1002*)
		proprietary_drivers_purge
		echo "No amd/ati driver for ZXE+3.0, use Radeon driver. " >> $LOG_VIDEO_SWITCH
		;;

		#is zxa/zxc/zxc+
		1d17*)
		install_ZXE_30_s3g_drv
		;;

		#other,will purge nonfree drivers.
		*)
		other_card_purge_drv
		;;
	esac
}
#end ZXE

################################# main ###########################################################
#第一步:2017-09 通过/proc/cpuinfo中的vendor_id来区分兆芯CPU和通用x86 CPU。兆芯CPU vendor_id :shanghai 或 CentaurHauls
#第二步：通过/proc/cpuinfo中的model name来区分ZX平台类型。ZXA：VIA Nano X2; ZXC:C-QuadCore; ZXC+:ZHAOXIN KaiXian ZX-C+; ZXD:ZHAOXIN KaiXian ZX-D.
#第三步:确定特定兆芯平台类型的机器类型。ZXA：只有台式机；ZXC：包括台式机、AIO一体机、CF03笔记本、CF03-T笔记本和miniPC；ZXD：目前只有台式机.
#第四步:确定系统版本。目前系统版本为：1.5,2.0和3.0.以内核版本区分.
#第五步:根据机型和系统版本，调用对应的sh，安装对应的驱动.

CPU_VENDOR_ID=`grep -i "vendor_id" /proc/cpuinfo | sort | uniq | awk -F ":" '{print $2}' | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g'`
CPU_MODEL_NAME=`grep -i "model name" /proc/cpuinfo | sort | uniq | awk -F ":" '{print $2}' | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g'`

#2019年起,ZXE平台及以后平台采用判定model方式来确定
#此发在早期的ZXD平台有些不适用，所以保留了早期对兆芯D\C\A平台的判定方法
#兆芯E平台的cpuinfo的model
#bit [5:4]为3, bit[3:0]为B
CPUMODEL_DEC=`cat /proc/cpuinfo | grep -v "model name" | grep "model" | awk -F : '{print $2}' | sed 's/ //g' | sort -u `
CPUMODEL_HEX=`echo 'obase=16; '$CPUMODEL_DEC'' | bc`
#取得model低4位 bit[3:0]
CPUMODEL=`printf "%x" $((0x$CPUMODEL_HEX&0x0F))`
CPUMODEL_EX_TEMP=`printf "%x" $((0x$CPUMODEL_HEX&0x30))`
#取得model,bit [5:4]
CPUMODEL_EX=`printf "%x" $((0x$CPUMODEL_EX_TEMP >> 4))`


if [ "${CPU_VENDOR_ID}" = "CentaurHauls" -o "${CPU_VENDOR_ID}"  = "Shanghai" ];then
	#ZX平台

    #先判定是否为ZXE,是则进行ZXE操作，否则执行早期代码
    #此处若有问题，可能是大写或平台硬件信息问题
    if [ "${CPUMODEL_EX}" = "3" -a "${CPUMODEL}" = "b" ] ;then
	#For ZXE
		case $KERNEL_VERSION in
			4.9*)
			#OS 3.1
			ZXE_30_install_drv
			;;
			*)
			echo "other kernel version" >> $LOG_VIDEO_OTHER
			;;
		esac
	#end ZXE
    else
	#确定ZX平台类型 (此处为2019年之前出的平台，ZXD、ZXC\ZXA等)
	case $CPU_MODEL_NAME in
		VIA?Nano?X2*)
		#ZXA平台。只有台式机，无其他机型
		case $KERNEL_VERSION in
			3.19*)
			#OS 1.5
			ZXA_15_install_drv
			;;
			4.3*)
			#OS 2.0
			ZXA_20_install_drv
			;;
			4.9*)
			#OS 3.0
			ZXA_30_install_drv
			;;
			*)
			echo "ZXA ,other kernel version" >> $LOG_VIDEO_OTHER
			;;
		esac
		;;
		C-QuadCore*)
		#ZXC平台。目前机型分为：台式机，一体机，CF03笔记本、CF03-T笔记本和miniPC。
		case $KERNEL_VERSION in
			3.19*)
			#OS 1.5
			ZXC_15_install_drv
			;;
			4.3*)
			#OS 2.0
			ZXC_20_install_drv
			;;
			4.9*)
			#OS 3.0
			ZXC_30_install_drv
			;;
			*)
			echo "ZXC, other kernel version" >> $LOG_VIDEO_OTHER
			;;
		esac
		;;
		ZHAOXIN*ZX-C*)
		#ZXC+平台。目前只有台式机，无其他机型。若有其他机型，在此处增加。
		case $KERNEL_VERSION in
			3.19*)
			#OS 1.5
			ZXCplus_15_install_drv
			;;
			4.3*)
			#OS 2.0
			ZXCplus_20_install_drv
			;;
			4.9*)
			#OS 3.0
			ZXCplus_30_install_drv
			;;
			*)
			echo "ZXC+ other kernel version" >> $LOG_VIDEO_OTHER
			;;
		esac
		;;
		ZHAOXIN*ZX-D*)
		#ZXD平台
		case $KERNEL_VERSION in
			3.19*)
			#OS 1.5
			ZXD_15_install_drv
			;;
			4.3*)
			#OS 2.0
			ZXD_20_install_drv
			;;
			4.9*)
			#OS 3.0
			ZXD_30_install_drv
			;;
			*)
			echo "ZXD other kernel version" >> $LOG_VIDEO_OTHER
			;;
		esac
		;;


		ZHAOXIN*KaiXian*KX-*)
		#ZXD平台,2018-1新机型台式机器 CPU信息:ZHAOXIN KaiXian KX-5640@2.0GHz
		case $KERNEL_VERSION in
			3.19*)
			#OS 1.5
			ZXD_15_install_drv
			;;
			4.3*)
			#OS 2.0
			ZXD_20_install_drv
			;;
			4.9*)
			#OS 3.0
			ZXD_30_install_drv
			;;
			*)
			echo "ZXD other kernel version" >> $LOG_VIDEO_OTHER
			;;
		esac
		;;



		*)
			echo "other ZX, can't detect!" >> $LOG_VIDEO_SWITCH
		;;
	esac
	#end 2019.1之前的平台.ZXA\ZXD\ZXC
    fi #end ZXE detect [ "${CPUMODEL_EX}" = "3" -a "${CPUMODEL}" = "b" ]
else
	#通用X86平台
	echo "Generic X86 CPU,do nothing." >> $LOG_VIDEO_SWITCH
fi #end CPU_VENDOR_ID
echo "Switch end ..." >> $LOG_VIDEO_SWITCH
exit 0
########## end main ##############
